@@ -1,4 +1,4 @@
-// $Id: main_thread.c,v 1.11 2006/03/28 20:00:54 khlut Exp $
+// $Id: main_thread.c,v 1.17 2006/04/27 16:34:05 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -21,6 +21,8 @@
 #include <sys/wait.h>
 #include <signal.h>
 #include <time.h>
+#include <unistd.h>
+#include <errno.h>
 
 #define LICENSE "Copyright 2005 Elphel, Inc.<br>"\
 "This plug-in is free software; you can redistribute it and/or modify"\
@@ -31,8 +33,9 @@
 #define Pd ((PD*)instance->pdata)
 
 NPNetscapeFuncs NPN;
-NPIdentifier ready,	run;
+NPIdentifier ready,	run,	src;
 int debug=1;
+int mozillastreams=1;
 static int instance_cnt;
 
 /*
@@ -109,8 +112,9 @@ int do_send_value(PD*pd, char *key, char *value) {
 void send_value(PD *pd, char *key, char *value) //send variable to slave
 {
   PR_Lock(pd->lock);
+  NPIdentifier keyid=NPN.getstringidentifier(key);
   if(( (!pd->slavectrl) || do_send_value(pd,key,value)<=0 ) && pd->status==Running // try to reanimate slave
-     && ( strcmp(key,"run") || strcmp(value,"0") )) { 	// ignore run=0 when slave is nonfunctional
+     && ( keyid!=run || strcmp(value,"0") )) { 	// ignore run=0 when slave is nonfunctional
     if(debug)perror("write to pipe");
    
     PR_Unlock(pd->lock);
@@ -125,18 +129,19 @@ void send_value(PD *pd, char *key, char *value) //send variable to slave
     // send all current variables
     int i;
     for(i=0;i<pd->vars->size;i++) {
-	if(pd->vars->var[i].key==run || pd->vars->var[i].key==ready) continue;
+	if(pd->vars->var[i].key==run || pd->vars->var[i].key==ready || pd->vars->var[i].key==keyid) continue;
 	char *sv=NPVariant_to_string(&(pd->vars->var[i].value));
 	do_send_value(pd, NPN.utf8fromidentifier(pd->vars->var[i].key), sv);
 	free(sv);
     }
     pd->xid && fprintf(pd->slavectrl,"xid=%x\n",(unsigned)pd->xid);
     do_send_value(pd,key,value);
+    fprintf(pd->slavectrl,".\n"); // end of parameters
     pd->status=Running;
     
     return;
     
-  }else if(pd->status==Running && !strcmp(key,"run") && !strcmp(value,"0") && pd->slavectrl) {
+  }else if(pd->status==Running && keyid==run && !strcmp(value,"0") && pd->slavectrl) {
     PR_Unlock(pd->lock);
     wait_slave(pd);
     pd->status=Running;
@@ -258,6 +263,7 @@ bool NPSetProperty(NPObject *npobj, NPIdentifier name, const NPVariant *value)
 
 char *NP_GetMIMEDescription(void)
 {
+  configure();
   Debug("NP_GetMIMEDescription\n");
   return get_mime_list();
 }
@@ -286,6 +292,7 @@ NPError NP_Initialize(NPNetscapeFuncs * moz_funcs, NPPluginFuncs * plugin_funcs)
 {
   void *ServiceManager=0;
   
+  configure();
   Debug("NP_Initialize\n");
   if (moz_funcs == NULL || plugin_funcs == NULL)
     return NPERR_INVALID_FUNCTABLE_ERROR;
@@ -300,6 +307,7 @@ NPError NP_Initialize(NPNetscapeFuncs * moz_funcs, NPPluginFuncs * plugin_funcs)
   NPN= *moz_funcs;
   ready=   NPN.getstringidentifier("ready");
   run=     NPN.getstringidentifier("run");
+  src=     NPN.getstringidentifier("src");
   instance_cnt=0;
   
   NPN.getvalue(0, NPNVserviceManager, &ServiceManager);
@@ -312,14 +320,14 @@ NPError NP_Initialize(NPNetscapeFuncs * moz_funcs, NPPluginFuncs * plugin_funcs)
   plugin_funcs->setwindow = NPP_SetWindow;
   plugin_funcs->destroy = NPP_Destroy;
   plugin_funcs->newstream = NPP_NewStream;
+  plugin_funcs->destroystream = NPP_DestroyStream;
+  plugin_funcs->writeready = NPP_WriteReady;
+  plugin_funcs->write = NPP_Write;
+  plugin_funcs->urlnotify = NPP_URLNotify;
+  plugin_funcs->print = NPP_Print;
 //	functions undefined
-//  plugin_funcs->destroystream = NPP_DestroyStream;
 //  plugin_funcs->setvalue = NPP_SetValue;
-//  plugin_funcs->writeready = NPP_WriteReady;
-//  plugin_funcs->write = NPP_Write;
 //  plugin_funcs->asfile = NPP_StreamAsFile;
-//  plugin_funcs->print = NPP_Print;
-//  plugin_funcs->urlnotify = NPP_URLNotify;
 //  plugin_funcs->event = NPP_HandleEvent;
 
   return NPERR_NO_ERROR;
@@ -357,6 +365,9 @@ NPError NPP_New(NPMIMEType mime, NPP instance, uint16 mode, int16 argc,
     pd->lock=PR_NewLock();
     pd->status=Init;
     pd->mime=strdup(mime);
+    pd->stream=0;
+    pd->npstream=0;
+    
     run_slave(pd,mime,0);	// run slave parallel process
     
     // slave observer thread
@@ -372,6 +383,7 @@ NPError NPP_New(NPMIMEType mime, NPP instance, uint16 mode, int16 argc,
      if(baseuri) {  send_value(pd,"baseURI",baseuri);  free(baseuri);  }
     }
     for(i=0; i<argc; i++) send_value(pd,argn[i],argv[i]);
+    fprintf(pd->slavectrl,".\n"); // end of parameters
     pd->status=Running;
     
     Debug("%p refcnt=%d\n",pd, Pd->refcnt);
@@ -413,7 +425,7 @@ void wait_slave(PD * pd) { // wait for slave termination
     PR_Sleep(100);
     process_pending_events(pd);
     Debug("wait %d\n",pd->slave);
-    if(waitpid(pd->slave,&i,WNOHANG)<0) perror("wait");
+    if(waitpid(pd->slave,&i,WNOHANG)<0 && debug) perror("wait");
     Debug("wait observer\n");
     if(pd->observer) {PR_JoinThread(pd->observer); pd->observer=0;}
     if(pd->slavectrl) {fclose(pd->slavectrl); pd->slavectrl=0;}
@@ -468,7 +480,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
            (int)npwindow->ws_info, (int)npwindow->type );
     
     if(Pd->xid!=xid) {
-    //TODO: display, type
+    //TODO: display
 	send_value_hex(Pd,"xid", (int)xid);
 	if(!Pd->xid) send_value_hex(Pd,"run", 1);
 	Pd->xid=xid;
@@ -478,8 +490,53 @@ NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
 
 NPError NPP_NewStream(NPP instance, NPMIMEType type, NPStream* stream, NPBool seekable, uint16* stype)
 {
-    Debug("%p NPP_NEWSTREAM %x\n", instance->pdata, (int)seekable);
-    return NPERR_GENERIC_ERROR;
+    Debug("%p NPP_NewStream %s %x %d\n", instance->pdata, stream->url, (int)seekable, (int)(*stype));
+    if(mozillastreams) {
+	Pd->npstream=stream;
+	return NPERR_NO_ERROR;
+    }
+    else return NPERR_GENERIC_ERROR;
+}
+
+NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
+{
+    Debug("%p NPP_DestroyStream %d\n", instance->pdata, reason);
+ 
+    if(Pd->npstream)
+      if(reason != NPRES_DONE) {
+	if(Pd->stream)close(Pd->stream);
+	Pd->stream=0;
+      }else {
+	PR_Lock(Pd->lock);
+	if(Pd->slavectrl)fprintf(Pd->slavectrl,"EOF\n");
+	PR_Unlock(Pd->lock);
+      }
+    Pd->npstream=0;
+    return NPERR_NO_ERROR;
+}
+int32   NPP_WriteReady(NPP instance, NPStream *stream)
+{
+    Debug("%p %d NPP_WriteReady\n", instance->pdata, (int)time(0));
+    if(!Pd->stream) return 0;
+    NPVariant *var=storage_get(Pd->vars, run);
+    if(!var || var->type!=NPVariantType_String || var->value.stringValue.utf8characters[0]!='1') return 0;
+    return 16384;
+}
+int32   NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
+{
+    Debug("%p NPP_Write\n", instance->pdata);
+    int i= write(Pd->stream,buffer,len);
+    Debug("written %d\n",i);
+    if(i<=0) return errno==EAGAIN?0:-1;
+    return i;
+}
+void    NPP_Print(NPP instance, NPPrint* printInfo)
+{
+    Debug("%p NPP_Print\n", instance->pdata);
+}
+void    NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
+{
+    Debug("%p NPP_URLNotify\n", instance->pdata);
 }
 
 /*
@@ -504,8 +561,8 @@ PLEvent * strevent( PD *pd, const char * str)  // make event from a string (is c
     ObserveEvent *event = PR_MALLOC(sizeof(ObserveEvent));
     if (!event)		{ PR_Unlock(pd->lock); return 0; }
 
-    if(strncmp(str,"debug=",6)==0) debug=str[6]-'0';
-    Debug("%p %d newEvent %s\n", pd, time(0), str);
+    //if(strncmp(str,"debug=",6)==0) debug=str[6]-'0';
+    Debug("%p %d newEvent %s\n", pd, (int)time(0), str);
     
     PL_InitEvent( &event->e, pd, HandleEvent, DestroyEvent);
     event->str=PR_Malloc(strlen(str)+3);
@@ -531,24 +588,32 @@ void PR_CALLBACK DestroyEvent(ObserveEvent* event)
 void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 {
     Debug("%p HandleEvent \"%s\"\t", event->pd, event->str+2);
-    size_t namelen;
-    for( namelen=2 ; ; namelen++ ) {
-    if( !event->str[namelen] ) { Debug("I\n");		return 0; }
-    else if( event->str[namelen] == ':'  ||  event->str[namelen] == '=' ) break;
-	else if( event->str[namelen] & 128 ) event->str[namelen]=' ';
-    }
-    event->str[namelen]=0;
-    char *name=strrchr(event->str+2,' ');
-    if(!name++) name=event->str+2;
-    if(!*name) {
-	Debug("I\n");		return 0;
+
+    char *name= event->str+2;
+    char *value= pair(&name, " \t", "=:\n", "\n");
+    if(!value || !*name) {
+	if(strcmp(event->str+2,"CLOSE")==0) {
+	    if(event->pd->stream)close(event->pd->stream);
+	    event->pd->stream=0;
+	}else if(event->pd->npstream && strcmp(event->str+2,"DESTROY")==0) {
+	    NPStream *stream=event->pd->npstream; event->pd->npstream=0;
+	    NPN.destroystream(event->pd->instance, stream, NPRES_DONE);
+	}else if(strcmp(event->str+2,"OPEN")==0) {
+	    char *srcv=NPVariant_to_string(storage_get(event->pd->vars, src));
+	    if(!srcv) { Debug("Bad src\n"); return 0;}
+	    Debug("geturl %s\n",srcv);
+	    NPN.geturl(event->pd->instance, srcv , 0);
+	    free(srcv);
+	}else Debug("I\n");
+	return 0;
     }
+
     NPVariant val;
-    STRINGZ_TO_NPVARIANT( event->str+namelen+1, val );
-    NPObject *obj; 
+    STRINGZ_TO_NPVARIANT( value, val );
+    NPObject *obj;
     if(event->pd->instance && NPN.getvalue(event->pd->instance, NPNVPluginElementNPObject, &obj)==NPERR_NO_ERROR) {
 	if( storage_set( &(event->pd->vars), NPN.getstringidentifier(name), &val ) ) {
-	    if(strcmp(name,"debug")==0) debug=event->str[namelen+1]-'0';
+	    //if(strcmp(name,"debug")==0) debug=*value-'0';
 	    // invoke a callback function
 	    name-=2; name[0]='o'; name[1]='n';
 	    int i;
@@ -562,8 +627,8 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 		)&&( Debug("getprop "), NPN.getproperty(event->pd->instance, obj, id0, &val) && NPVARIANT_IS_STRING(val)
 	         || (NPN.releasevariantvalue(&val), NPN.getproperty(event->pd->instance, obj, id1, &val) && NPVARIANT_IS_STRING(val))
 		)
-	    ) {
-		Debug("evaluate ");
+	    ) { // this is not function but string
+		//Debug("evaluate ");
 		//In "evaluate" the instance can be desroyed
 		//NPN.evaluate(event->pd->instance, obj, &(val.value.stringValue), 0);
 		char *url=PR_Malloc(val.value.stringValue.utf8length+12);
