@@ -1,4 +1,4 @@
-// $Id: observer_thread.c,v 1.5 2006/03/22 21:02:09 khlut Exp $
+// $Id: observer_thread.c,v 1.6 2006/04/15 09:28:58 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -22,32 +22,36 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
+#include <fcntl.h>
 
 int run_slave( PD * pd, char * mime, int restart ) // this function is not in observer thread, it run a separate parallel process
 {
-    int p[2]={-1,-1};
+    int p[2]={-1,-1}, st[2]={-1,-1};
     char *path = find_slave(mime);
     
     if(socketpair(AF_UNIX, SOCK_STREAM, 0, p)) perror("socketpair");
+    if(pipe(st)) perror("pipe");
     fflush(stdout);
     pd->slave=fork();
     if(!pd->slave) { // new process branch
-	close(p[0]);
-	dup2(p[1],0);
+	close(p[0]);	close(st[1]);
+	dup2(p[1],0);	dup2(st[0],3);
 	dup2(p[1],1);
 	dup2(p[1],2);
-	close(p[1]);
+	close(p[1]);	close(st[0]);
 	if(restart) execl(path,path,"--no-vars",0);
 	else execl(path,path,0);
 	perror("error");
-	Debug("errorfile=%s\n",path);
-	Debug("run=0\n");	
+	printf("errorfile=%s\n",path);
+	printf("run=0\n");	
 	_exit(1);
     }
     if(pd->slave==-1) return 1;
-    close(p[1]);
+    close(p[1]);	close(st[0]);
     pd->slavectrl=fdopen(p[0],"w");
     setlinebuf(pd->slavectrl);
+    pd->stream=st[1];
+    if(fcntl(st[1],F_SETFL,O_NONBLOCK)<0) perror("fcntl nonblock");
     return 0;
 }
 
