@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# $Id: mplayer.pl,v 1.22 2006/03/26 10:32:46 khlut Exp $
+# $Id: mplayer.pl,v 1.28 2006/04/27 16:34:05 khlut Exp $
 # Bind mplayer and mencoder with HTML object
 # ElphelOgm is used to receive and record rtp mjpeg video
 #
@@ -51,6 +51,8 @@ load_config();
 
 $v{quittimeout}=2; #wait 2 seconds after send quit command or interruptable signal
 $v{play}=1;
+$loop=1;
+$href="/dev/fd/3";
 
 $|=1; #forces a flush right away and after every write or print on the currently selected output channel
 my $shutstate:shared=0;
@@ -73,7 +75,7 @@ sub progress {
     my $W=Gtk2::Gdk::Window->foreign_new(hex $v{xid});
     my (undef,undef,$w,$h) = $W->get_geometry;
     my $gc=Gtk2::Gdk::GC->new($W);
-    $W->draw_rectangle ($gc, 1 , 0, 0, $w*$_[0]/10, 10);
+    $W->draw_rectangle ($gc, 1 , 0, 0, $w*$_[0]/50, 10);
     Gtk2::Gdk->flush;
 }
 
@@ -120,50 +122,77 @@ sub normal_exit{
 $SIG{PIPE}=\&normal_exit;
 $SIG{ALRM}=\&shut;
 
-
-sub ifilt{
-    if($v{pause} || !$v{play}) {
-	$pausestate=1;
-	print SLVW "pause\n";
-	lock $queue;
-	$queue.="pause=1\nplay=0\n";
+sub url2mplayer {
+    $v{href}=$v{baseURI}.'/'.$v{href}	if( $v{href}!~m|[a-zA-Z0-9]+:| );
+    $href=$v{href};
+    if($ENV{GENRES_MOZILLA_STREAMS} && $v{src} && $href=~m!^(http.?|ftp.?)://!) {
+	$href="/dev/fd/3";
+    }else{
+	$href=~s|^file://||;
     }
-    while(<STDIN>){
-	($var,$val)= split '=', $_, 2;
-	$val=~s/\n//;
-	if( $_ eq "run=0\n" ) {
-	    shut();
-	    last;
-	}elsif( $var eq "pause" ) {
-	    $val=0+$val;
-	    $val=1 if $val!=0 && $val!=1 ;
-	    if($val != $pausestate) {
-		$pausestate=$val;
-		print SLVW "pause\n";
-		lock $queue;
-		$queue.="pause=$val\nplay=".(1-$val)."\n";
-	    }	    
-	}elsif( $var eq "filepos" ) {
-	    if($val =~ s/%//) {
-		print SLVW "seek $val 1\n";
-	    }else{
-		print SLVW "seek $val 2\n";
-	    }
-	}elsif( $var eq "framestep" ) {
-	    print SLVW "frame_step\n";	    
-	    $pausestate=1;
-	    lock $queue;
-	    $queue.="pause=1\nplay=0\n";
-	}elsif( $var eq "src" or $var eq "href") {
-	    $v{href}=$val;
-	    print SLVW "loadfile $val\n";
-	    lock $queue;
-	    $queue.="$_";
-	}else{
-	    $v{$var}=$val;
-	    lock $queue;
-	    $queue.="$_";
+}
+sub cmd {
+    my ($c)=@_;
+    my ($var,$val) = split '=', $c, 2;
+    $val=~s/\n//;
+    if( $c eq "run=0\n" ) {
+        shut();
+        return;
+    }elsif( $c eq "EOF\n" ) {
+	lock $queue;
+	if($href eq "/dev/fd/3" && ($loop==0 || $loop>1)) {
+    	    print "OPEN\n";
+	    $loop-- if($loop);
+        }else{
+    	    print "CLOSE\n";
 	}
+    }elsif( $var eq "pause" ) {
+        $val=0+$val;
+        $val=1 if $val!=0 && $val!=1 ;
+        if($val != $pausestate) {
+    	    $pausestate=$val;
+    	    print SLVW "pause\n";
+    	    lock $queue;
+    	    $queue.="pause=$val\nplay=".(1-$val)."\n";
+	}	    
+    }elsif( $var eq "filepos" ) {
+        if($val =~ s/%//) {
+    	    print SLVW "seek $val 1\n";
+        }else{
+	    print SLVW "seek $val 2\n";
+        }
+    }elsif( $var eq "framestep" ) {
+        print SLVW "frame_step\n";	    
+        $pausestate=1;
+        lock $queue;
+        $queue.="pause=1\nplay=0\n";
+    }elsif( $var eq "src" or $var eq "href") {
+        $v{$var}=$val;
+        $v{href}=$val;
+	my $prhref=$href;
+        url2mplayer();
+        print SLVW "loadfile $href\n" if $href ne $prhref || $href ne '/dev/fd/3';
+        lock $queue;
+        $queue.="src=$val\nhref=$val\n";
+	if($prhref eq '/dev/fd/3') {
+	    $queue.="DESTROY\n"; # destroy old stream
+	}
+	if($href eq '/dev/fd/3') {
+	    $queue.="OPEN\n"; # destroy old stream	    
+	}
+    }else{
+        $v{$var}=$val;
+        lock $queue;
+        $queue.="$c";
+    }
+}
+sub ifilt{
+    
+    cmd("pause=1") 		    if $v{pause} || !$v{play};
+    cmd("filepos=$v{filepos}")      if $v{filepos};
+    cmd("framestep=$v{framestep}")  if $v{framestep};
+    while(<STDIN>){
+	cmd $_;
     }
 }
 
@@ -190,7 +219,8 @@ sub ofilt{
 	    $queue="";
 	  }
 	}
-	if($_ eq "\n") {
+	if($_ eq "\n") {{
+	    lock $queue;
 	    if( $s =~ 'Failed' ) {
 	        print "error=$s";
 	    }elsif($s =~ m/^Playing (.*)\.$/){
@@ -198,6 +228,8 @@ sub ofilt{
 		if($s !~ m!-|/dev!) {
 		    $s="file://".$s if $s !~ m|^[a-z0-9]*://|;
 		    print "url=$s";
+		}else{
+		    print "url=$v{href}\n";
 		}
 	    }elsif($s =~ m/^Cache fill:[[:space:]]*([0-9.]*)%/){
 		progress($1);
@@ -211,7 +243,7 @@ sub ofilt{
 		print $s;
 	    }
 	    $s="";
-	}
+	}}
       }
     }while($restart && !$shutstate && $restart && &$restart);
     ofilt_quit() unless $SIG{ALRM} eq \&shut;;
@@ -250,6 +282,7 @@ print "debug=1\n", # list of control variables
       "pause=0\n",
       "framestep=\n",
       "filepos=\n",
+      "autostart=\n",
       "play=1\n",
       "save=\n",
       "fps=\n",
@@ -259,19 +292,27 @@ print "debug=1\n", # list of control variables
       
 while (<STDIN>){
     s/\n//;
+    if( $_ eq '.' ) {
+      if($v{href} && $v{autostart} !~ m/^(|0|false)$/i){  ##end of parameters marker
+	 $_="run=1";
+      }else{next;}
+    }
     if($_ !~ /.*=/){
 	print "error=1\n";
 	next;
     }
     s/'/'\\''/g; #'
     ($x,$y) = split '=', $_, 2;
-    $x='href' if($x eq 'src' || $x eq 'SRC');
-    $v{$x}=$y;
+    if($x eq 'src' || $x eq 'SRC') {
+	$v{src}=$y;
+	$v{$x='href'}=$y;
+    }else{
+	$v{$x}=$y;
+    }
     if("$_" eq "run=1"){
 	$OPTS="";
 	$v{baseURI}=~s|/[^/]*$||;
-	$v{href}=$v{baseURI}.'/'.$v{href}	if( $v{href}!~m|[a-zA-Z0-9]+:| );
-	$v{href}=~s|^file://||;
+	url2mplayer();
 	if($v{save}) { #Recording TODO: ask about overwriting file
 	    $cmd=rtpcmd();
 	    if($cmd) {
@@ -282,12 +323,12 @@ while (<STDIN>){
 		$cfg{out_file}[0]=~s/\.ogm/\.avi/;
 	    }
 	    if(uc $v{save} ne 'AUTO'){
-		if(!file_dialog(out_file_once, frames)) { print "run=0"; last};
+		if(!file_dialog(out_file_once, frames)) { print "run=0\n"; last};
 		$out_file=$cfg{out_file_once}[0];
 	    }elsif( $cfg{not_ask}[0] ){
 		$out_file=$cfg{out_file}[0];
 	    }else{
-		if(!file_dialog(out_file, not_ask, frames)) { print "run=0"; last};
+		if(!file_dialog(out_file, not_ask, frames)) { print "run=0\n"; last};
 		$out_file=$cfg{out_file}[0];
 	    }
 	    if($cmd) {
@@ -309,8 +350,9 @@ while (<STDIN>){
 	    $OPTS.=" -xy '$v{width}'"	if( $v{width} && $v{width}!~"%" );
 	    $OPTS.=" -lavdopts lowres='$v{lowres}'" if( $v{lowres} );
 	    $OPTS.=" -fps '$v{fps}'"		if( $v{fps} );
-	    $OPTS.=" -fixed-vo -loop ".(0+$v{loop}) if( exists $v{loop} );
-	    $OPTS.= localnet()? " -nocache" : " -cache 1000 -cache-min 10";
+	    $OPTS.=" -fixed-vo -loop ".(0+$v{loop}) if( $href ne "/dev/fd/3" && exists $v{loop} && "\L$v{loop}" ne 'false' );
+	    $loop= 0+$v{loop} if( exists $v{loop} && "\L$v{loop}" ne 'false' );
+	    $OPTS.= localnet()? " -nocache" : " -cache 1000 -cache-min 50";
 	    if(( $cmd2=rtpcmd() )) {
 		$cmd2.=" -k $v{frameskip}" if $v{frameskip};
 	        $cmd2="|exec $cmd2 -i 2>/dev/null ".fileno(PIPE_OUT)."<&- >/dev/fd/".fileno(PIPE_IN);
@@ -321,7 +363,7 @@ while (<STDIN>){
 		$sl2p=open(SL2W,$cmd2);
 	    } else {
 		$OPTS.=" -ao $cfg{ao}[0]" if $cfg{ao}[0];
-		$cmd="mplayer $OPTS -vo $cfg{vo}[0] -zoom -nojoystick -slave -nomouseinput -cookies '$v{href}'";
+		$cmd="mplayer $OPTS -vo $cfg{vo}[0] -zoom -nojoystick -slave -nomouseinput -cookies '$href'";
 	        print "cmd=$cmd\n";
 	        runslv $cmd;
 	    }
